@@ -192,6 +192,7 @@ fn substitute_macros(input: &str) -> String {
         ("[CHECKING]", "    Checking"),
         ("[COMPLETED]", "   Completed"),
         ("[CREATED]", "     Created"),
+        ("[CREATING]", "    Creating"),
         ("[CREDENTIAL]", "  Credential"),
         ("[DOWNGRADING]", " Downgrading"),
         ("[FINISHED]", "    Finished"),
@@ -206,6 +207,8 @@ fn substitute_macros(input: &str) -> String {
         ("[UPDATING]", "    Updating"),
         ("[ADDING]", "      Adding"),
         ("[REMOVING]", "    Removing"),
+        ("[REMOVED]", "     Removed"),
+        ("[UNCHANGED]", "   Unchanged"),
         ("[DOCTEST]", "   Doc-tests"),
         ("[PACKAGING]", "   Packaging"),
         ("[PACKAGED]", "    Packaged"),
@@ -235,6 +238,8 @@ fn substitute_macros(input: &str) -> String {
         ("[SKIPPING]", "    Skipping"),
         ("[WAITING]", "     Waiting"),
         ("[PUBLISHED]", "   Published"),
+        ("[BLOCKING]", "    Blocking"),
+        ("[GENERATED]", "   Generated"),
     ];
     let mut result = input.to_owned();
     for &(pat, subst) in &macros {
@@ -588,15 +593,24 @@ fn find_json_mismatch_r<'a>(
                 .next()
         }
         (&Object(ref l), &Object(ref r)) => {
-            let same_keys = l.len() == r.len() && l.keys().all(|k| r.contains_key(k));
-            if !same_keys {
-                return Some((expected, actual));
-            }
+            let mut expected_entries = l.iter();
+            let mut actual_entries = r.iter();
 
-            l.values()
-                .zip(r.values())
-                .filter_map(|(l, r)| find_json_mismatch_r(l, r, cwd))
-                .next()
+            loop {
+                match (expected_entries.next(), actual_entries.next()) {
+                    (None, None) => return None,
+                    (Some((expected_key, expected_value)), Some((actual_key, actual_value)))
+                        if expected_key == actual_key =>
+                    {
+                        if let mismatch @ Some(_) =
+                            find_json_mismatch_r(expected_value, actual_value, cwd)
+                        {
+                            return mismatch;
+                        }
+                    }
+                    _ => return Some((expected, actual)),
+                }
+            }
         }
         (&Null, &Null) => None,
         // Magic string literal `"{...}"` acts as wildcard for any sub-JSON.
