@@ -71,16 +71,16 @@ use crate::ops;
 use crate::sources::PathSource;
 use crate::util::cache_lock::CacheLockMode;
 use crate::util::errors::CargoResult;
-use crate::util::{profile, CanonicalUrl};
-use crate::util_schemas::manifest::RustVersion;
+use crate::util::CanonicalUrl;
 use anyhow::Context as _;
+use cargo_util_schemas::manifest::RustVersion;
 use std::collections::{HashMap, HashSet};
 use tracing::{debug, trace};
 
 /// Result for `resolve_ws_with_opts`.
-pub struct WorkspaceResolve<'cfg> {
+pub struct WorkspaceResolve<'gctx> {
     /// Packages to be downloaded.
-    pub pkg_set: PackageSet<'cfg>,
+    pub pkg_set: PackageSet<'gctx>,
     /// The resolve for the entire workspace.
     ///
     /// This may be `None` for things like `cargo install` and `-Zavoid-dev-deps`.
@@ -109,10 +109,8 @@ version. This may also occur with an optional dependency that is not enabled.";
 /// This is a simple interface used by commands like `clean`, `fetch`, and
 /// `package`, which don't specify any options or features.
 pub fn resolve_ws<'a>(ws: &Workspace<'a>) -> CargoResult<(PackageSet<'a>, Resolve)> {
-    let max_rust_version = ws.rust_version();
-
-    let mut registry = PackageRegistry::new(ws.config())?;
-    let resolve = resolve_with_registry(ws, &mut registry, max_rust_version)?;
+    let mut registry = PackageRegistry::new(ws.gctx())?;
+    let resolve = resolve_with_registry(ws, &mut registry)?;
     let packages = get_resolved_packages(&resolve, registry)?;
     Ok((packages, resolve))
 }
@@ -127,32 +125,29 @@ pub fn resolve_ws<'a>(ws: &Workspace<'a>) -> CargoResult<(PackageSet<'a>, Resolv
 ///
 /// `specs` may be empty, which indicates it should resolve all workspace
 /// members. In this case, `opts.all_features` must be `true`.
-pub fn resolve_ws_with_opts<'cfg>(
-    ws: &Workspace<'cfg>,
-    target_data: &mut RustcTargetData<'cfg>,
+pub fn resolve_ws_with_opts<'gctx>(
+    ws: &Workspace<'gctx>,
+    target_data: &mut RustcTargetData<'gctx>,
     requested_targets: &[CompileKind],
     cli_features: &CliFeatures,
     specs: &[PackageIdSpec],
     has_dev_units: HasDevUnits,
     force_all_targets: ForceAllTargets,
-    max_rust_version: Option<&RustVersion>,
-) -> CargoResult<WorkspaceResolve<'cfg>> {
-    let mut registry = PackageRegistry::new(ws.config())?;
+) -> CargoResult<WorkspaceResolve<'gctx>> {
+    let mut registry = PackageRegistry::new(ws.gctx())?;
     let mut add_patches = true;
     let resolve = if ws.ignore_lock() {
         None
     } else if ws.require_optional_deps() {
         // First, resolve the root_package's *listed* dependencies, as well as
         // downloading and updating all remotes and such.
-        let resolve = resolve_with_registry(ws, &mut registry, max_rust_version)?;
+        let resolve = resolve_with_registry(ws, &mut registry)?;
         // No need to add patches again, `resolve_with_registry` has done it.
         add_patches = false;
 
         // Second, resolve with precisely what we're doing. Filter out
         // transitive dependencies if necessary, specify features, handle
         // overrides, etc.
-        let _p = profile::start("resolving with overrides...");
-
         add_overrides(&mut registry, ws)?;
 
         for (replace_spec, dep) in ws.root_replace() {
@@ -160,13 +155,13 @@ pub fn resolve_ws_with_opts<'cfg>(
                 .iter()
                 .any(|r| replace_spec.matches(r) && !dep.matches_id(r))
             {
-                ws.config()
+                ws.gctx()
                     .shell()
                     .warn(format!("package replacement is not used: {}", replace_spec))?
             }
 
             if dep.features().len() != 0 || !dep.uses_default_features() {
-                ws.config()
+                ws.gctx()
                 .shell()
                 .warn(format!(
                     "replacement for `{}` uses the features mechanism. \
@@ -190,7 +185,6 @@ pub fn resolve_ws_with_opts<'cfg>(
         None,
         specs,
         add_patches,
-        max_rust_version,
     )?;
 
     let pkg_set = get_resolved_packages(&resolved_with_overrides, registry)?;
@@ -239,10 +233,10 @@ pub fn resolve_ws_with_opts<'cfg>(
     })
 }
 
-fn resolve_with_registry<'cfg>(
-    ws: &Workspace<'cfg>,
-    registry: &mut PackageRegistry<'cfg>,
-    max_rust_version: Option<&RustVersion>,
+#[tracing::instrument(skip_all)]
+fn resolve_with_registry<'gctx>(
+    ws: &Workspace<'gctx>,
+    registry: &mut PackageRegistry<'gctx>,
 ) -> CargoResult<Resolve> {
     let prev = ops::load_pkg_lockfile(ws)?;
     let mut resolve = resolve_with_previous(
@@ -254,7 +248,6 @@ fn resolve_with_registry<'cfg>(
         None,
         &[],
         true,
-        max_rust_version,
     )?;
 
     if !ws.is_ephemeral() && ws.require_optional_deps() {
@@ -278,21 +271,21 @@ fn resolve_with_registry<'cfg>(
 ///
 /// If `register_patches` is true, then entries from the `[patch]` table in
 /// the manifest will be added to the given `PackageRegistry`.
-pub fn resolve_with_previous<'cfg>(
-    registry: &mut PackageRegistry<'cfg>,
-    ws: &Workspace<'cfg>,
+#[tracing::instrument(skip_all)]
+pub fn resolve_with_previous<'gctx>(
+    registry: &mut PackageRegistry<'gctx>,
+    ws: &Workspace<'gctx>,
     cli_features: &CliFeatures,
     has_dev_units: HasDevUnits,
     previous: Option<&Resolve>,
     to_avoid: Option<&HashSet<PackageId>>,
     specs: &[PackageIdSpec],
     register_patches: bool,
-    max_rust_version: Option<&RustVersion>,
 ) -> CargoResult<Resolve> {
     // We only want one Cargo at a time resolving a crate graph since this can
     // involve a lot of frobbing of the global caches.
     let _lock = ws
-        .config()
+        .gctx()
         .acquire_package_cache_lock(CacheLockMode::DownloadExclusive)?;
 
     // Here we place an artificial limitation that all non-registry sources
@@ -322,11 +315,11 @@ pub fn resolve_with_previous<'cfg>(
     // While registering patches, we will record preferences for particular versions
     // of various packages.
     let mut version_prefs = VersionPreferences::default();
-    if ws.config().cli_unstable().minimal_versions {
+    if ws.gctx().cli_unstable().minimal_versions {
         version_prefs.version_ordering(VersionOrdering::MinimumVersionsFirst)
     }
-    if ws.config().cli_unstable().msrv_policy {
-        version_prefs.max_rust_version(max_rust_version.cloned());
+    if ws.gctx().cli_unstable().msrv_policy {
+        version_prefs.max_rust_version(ws.rust_version().cloned().map(RustVersion::into_partial));
     }
 
     // This is a set of PackageIds of `[patch]` entries, and some related locked PackageIds, for
@@ -512,7 +505,8 @@ pub fn resolve_with_previous<'cfg>(
         &replace,
         registry,
         &version_prefs,
-        Some(ws.config()),
+        ResolveVersion::with_rust_version(ws.rust_version()),
+        Some(ws.gctx()),
     )?;
     let patches: Vec<_> = registry
         .patches()
@@ -528,20 +522,21 @@ pub fn resolve_with_previous<'cfg>(
     if let Some(previous) = previous {
         resolved.merge_from(previous)?;
     }
-    let config = ws.config();
-    let mut deferred = config.deferred_global_last_use()?;
-    deferred.save_no_error(config);
+    let gctx = ws.gctx();
+    let mut deferred = gctx.deferred_global_last_use()?;
+    deferred.save_no_error(gctx);
     Ok(resolved)
 }
 
 /// Read the `paths` configuration variable to discover all path overrides that
 /// have been configured.
+#[tracing::instrument(skip_all)]
 pub fn add_overrides<'a>(
     registry: &mut PackageRegistry<'a>,
     ws: &Workspace<'a>,
 ) -> CargoResult<()> {
-    let config = ws.config();
-    let Some(paths) = config.get_list("paths")? else {
+    let gctx = ws.gctx();
+    let Some(paths) = gctx.get_list("paths")? else {
         return Ok(());
     };
 
@@ -549,12 +544,12 @@ pub fn add_overrides<'a>(
         // The path listed next to the string is the config file in which the
         // key was located, so we want to pop off the `.cargo/config` component
         // to get the directory containing the `.cargo` folder.
-        (def.root(config).join(s), def)
+        (def.root(gctx).join(s), def)
     });
 
     for (path, definition) in paths {
         let id = SourceId::for_path(&path)?;
-        let mut source = PathSource::new_recursive(&path, id, ws.config());
+        let mut source = PathSource::new_recursive(&path, id, ws.gctx());
         source.update().with_context(|| {
             format!(
                 "failed to update path override `{}` \
@@ -568,10 +563,10 @@ pub fn add_overrides<'a>(
     Ok(())
 }
 
-pub fn get_resolved_packages<'cfg>(
+pub fn get_resolved_packages<'gctx>(
     resolve: &Resolve,
-    registry: PackageRegistry<'cfg>,
-) -> CargoResult<PackageSet<'cfg>> {
+    registry: PackageRegistry<'gctx>,
+) -> CargoResult<PackageSet<'gctx>> {
     let ids: Vec<PackageId> = resolve.iter().collect();
     registry.get(&ids)
 }
@@ -861,7 +856,7 @@ fn emit_warnings_of_unused_patches(
                 for id in ids.iter() {
                     write!(msg, "\n    {}", id.display_registry_name())?;
                 }
-                ws.config().shell().warn(msg)?;
+                ws.gctx().shell().warn(msg)?;
             }
             _ => unemitted_unused_patches.push(unused),
         }
@@ -873,7 +868,7 @@ fn emit_warnings_of_unused_patches(
             .iter()
             .map(|pkgid| format!("Patch `{}` {}", pkgid, MESSAGE))
             .collect();
-        ws.config()
+        ws.gctx()
             .shell()
             .warn(format!("{}\n{}", warnings.join("\n"), UNUSED_PATCH_WARNING))?;
     }
