@@ -862,7 +862,7 @@ fn main() {
             123..126 'S()': S<i32>
             132..133 's': S<i32>
             132..144 's.g(|_x| {})': ()
-            136..143 '|_x| {}': impl Fn(&i32)
+            136..143 '|_x| {}': impl FnOnce(&i32)
             137..139 '_x': &i32
             141..143 '{}': ()
             150..151 's': S<i32>
