@@ -1,5 +1,5 @@
 // rustfmt-edition: 2015
-#![feature(rust_2018_preview, uniform_paths)]
+#![feature(uniform_paths)]
 use futures::prelude::*;
 use http_03::cli::Cli;
 use hyper::{service::service_fn_ok, Body, Response, Server};
