@@ -1,11 +1,11 @@
 // Test explores how `#[structral_match]` behaves in tandem with
 // `*const` and `*mut` pointers.
 
-// run-pass
+//@ run-pass
 
 #![warn(pointer_structural_match)]
 
-struct NoDerive(#[allow(unused_tuple_struct_fields)] i32);
+struct NoDerive(#[allow(dead_code)] i32);
 
 // This impl makes NoDerive irreflexive
 // (which doesn't matter here because `<*const T>::eq` won't recur on `T`).
