@@ -1,11 +1,11 @@
 // This used to cause assert_10_13 to unexpectingly fail, due to simd_shuffle_indices being passed
 // the wrong Instance, causing issues with inlining. See #67557.
 //
-// run-pass
-// compile-flags: -Zmir-opt-level=4
-#![feature(platform_intrinsics, repr_simd)]
+//@ run-pass
+//@ compile-flags: -Zmir-opt-level=4
+#![feature(intrinsics, repr_simd)]
 
-extern "platform-intrinsic" {
+extern "rust-intrinsic" {
     fn simd_shuffle<T, I, U>(x: T, y: T, idx: I) -> U;
 }
 
