@@ -1,4 +1,4 @@
-// check-pass
+//@ check-pass
 //! The presence of an `align(X)` annotation must be accounted for.
 
 #![crate_type = "lib"]
@@ -7,11 +7,10 @@
 
 mod assert {
     use std::mem::{Assume, BikeshedIntrinsicFrom};
-    pub struct Context;
 
     pub fn is_maybe_transmutable<Src, Dst>()
     where
-        Dst: BikeshedIntrinsicFrom<Src, Context, {
+        Dst: BikeshedIntrinsicFrom<Src, {
             Assume {
                 alignment: true,
                 lifetimes: true,
